/************************************************************************
 *                                                                      *
 *   Ant Stratego                                                       *
 *   (C) 2000 Antoine Potten                                            *
 *   software@antp.be - http://www.antp.be/software                     *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 2     *
 *   of the License, or (at your option) any later version.             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   hist.cpp : Gestion de l'historique (affichage, enregistrement,     *
 *              chargement                                              *
 *                                                                      *
 ************************************************************************/

#include "Unit3.h"
#include "Hist.h"
#include <string.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include "Files.h"

#define JOUEUR_ROUGE true
#define JOUEUR_BLEU false

char szPiecesHist[13][15]={
/* Noms des pices affichs dans la barre d'tat */
     "",
     "Marchal (1)",
     "Gnral (2)",
     "Colonel (3)",
     "Commandant (4)",
     "Capitaine (5)",
     "Lieutenant (6)",
     "Sergent (7)",
     "Dmineur (8)",
     "claireur (9)",
     "Espion",
     "Bombe",
     "Drapeau"
};

void __fastcall THistorique::AddHistorique(bool bJoueur, int iOld, int jOld,
                    int iNew, int jNew, int iPiece, int iCapture)
{ /* Assemblage d'une chane qui sera place dans la barre d'tat et qui
     servira au joueur  connatre le dplacement de son adversaire ou
     le rsultat du combat (qu'il n'a pas pu voir)
     Si ncessaire, demande d'criture de ce dplacement dans le fichier
     d'historique */
  /* bJoueur = Joueur qui a effectu le dplacement
     iOld -> jNew = Coordonnes du dplacement
     iPiece = Rang de la pice dplace
     iCapture = Si < 0 la pice dplace a perdu, si > 0 elle a gagn,
                    Si = 0 les deux pices tombent, Si = 99 il s'agit d'un
                    dplacement simple. La valeur absolue de cette variable
                    donne le rang de la pice attaque */
     if(bComposer==false) Inverse(&iOld,&jOld,&iNew,&jNew);
          /* Le tableau est retourn  chaque joueur, il faut donc adapter
               les coordonnes */
     strcpy(szDepl,"Joueur ");
     char szTemp[256];
     switch(bJoueur) {
          case JOUEUR_ROUGE : strcat(szDepl,"rouge : "); break;
          case JOUEUR_BLEU  : strcat(szDepl,"bleu : " );break;
     }
     if(iCapture !=99) strcat(szDepl,szPiecesHist[iPiece%13]);
     sprintf(szTemp," [%i,%i] ",iOld,jOld);
     strcat(szDepl,szTemp);
     if(iCapture==99) sprintf(szTemp,"-> [%i,%i]",iNew,jNew);
     else if(iCapture<0) sprintf(szTemp," est pris par %s [%i,%i]",
               szPiecesHist[(-iCapture)%13],iNew,jNew);
     else if(iCapture==0) sprintf(szTemp," et %s [%i,%i] sont limins",
               szPiecesHist[iPiece%13],iNew,jNew);
     else sprintf(szTemp," a pris %s [%i,%i]",
               szPiecesHist[iCapture%13],iNew,jNew);
     strcat(szDepl,szTemp); /* szDepl = Variable  afficher par aprs dans
                                   la barre d'tat*/
     if(iType==HIST_ENREG) {
          /* Ajout d'un enregistrement dans le fichier */
          if(bJoueur==JOUEUR_ROUGE&&bComposer==false)
               Inverse(&iOld,&jOld,&iNew,&jNew);
               /* Pour le replay, le tableau sera toujours dans le mme sens */
          strDepl.iOld=iOld;
          strDepl.iNew=iNew;
          strDepl.jOld=jOld;
          strDepl.jNew=jNew;
          strDepl.iCapture=iCapture;
          if(write(iFile,&strDepl,sizeof(TDeplacement))<=0) {
               WriteError();
          }
     }
}

void __fastcall THistorique::Positions(int aiPositions[10][10])
{ /* Enregistrement dans le fichier d'historique des positions initiales
          des pions */
  /* aiPositions[][] = Tableau contenant les rangs des pions */
     if(iType==HIST_ENREG) {
          if(write(iFile,aiPositions,10*10*sizeof(int))<=0) {
               WriteError();
          }
     }
}

void __fastcall THistorique::Affiche()
{ /* Affiche la chane composs par AddHistorique() dans la barre d'tat */
     MainWindow->StatusBar1->SetTextBuf(szDepl);
}

__fastcall THistorique::THistorique(int iReplayType, char szHistFile[],
          bool bComp)
{ /* Constructeur : initialise l'historique d'aprs les options choisies */
  /* iReplayType = Dit s'il faut ou non enregistrer l'historique */
  /* szHistFile[] = Nom de fichier o enregistrer / lire l'historique */
     bComposer=bComp;
     szDepl[0]=0;
     strcpy(szFileName,szHistFile);
     if((iType=iReplayType)!=HIST_NOHIST) {
          /* Il faudra peut-tre enregistrer l'historique */
          if(iType==HIST_DEMAND) {
               /* Demander s'il faut enregistrer l'historique */
               switch(Application->MessageBox("Voulez-vous choisir un fichier "
                         "pour que l'historique de la partie y soit "
                         "enregistr  chaque dplacement ?","Question",
                         MB_YESNO|MB_ICONQUESTION)) {
                    case IDYES :
                         /* Choix du nom de fichier */
                         if(MainWindow->SDReplay->Execute()) {
                              /* Il faudra enregistrer l'historique */
                              strcpy(szFileName,
                                   MainWindow->SDReplay->FileName.c_str());
                              iType=HIST_ENREG;
                         } else iType=HIST_NOHIST;
                              /* Ne pas enregistrer */
                         break;
                    case IDNO :
                         /* Ne pas enregistrer */
                         iType=HIST_NOHIST;
                         break;
               }
          }
     }
     if(iType!=HIST_NOHIST) {
          /* Il faut l'enregistrer */
          if((iFile=open(szFileName,O_WRONLY|O_CREAT|O_TRUNC,
                    S_IREAD|S_IWRITE))==-1) {
                    /* Problme avec le fichier */
               Application->MessageBox("Impossible d'ouvrir le fichier pour "
                    "enregistrer l'historique","Erreur",MB_OK|MB_ICONSTOP);
               iType=HIST_NOHIST;
          } else {
               iType=HIST_ENREG;
               char szHeader[]=ASH_HEADER; // En-tte du fichier (header)
               if(write(iFile,&szHeader,sizeof(szHeader))<=0) {
                    WriteError();
               }
          }
     }
}

__fastcall THistorique::~THistorique()
{ /* Si on tait occup  enregistrer l'historique, il faut marquer la fin de
     fichier (non indispensable) et le fermer proprement */
     if(iType==HIST_ENREG) {
          char szEOF[]=ASH_EOFILE;
          if(write(iFile,&szEOF,sizeof(szEOF))<=0) {
               WriteError();
          }
     }
     close(iFile);
}

void __fastcall THistorique::WriteError()
{ /* Erreur d'criture -> On n'enregistre pas l'historique */
     Application->MessageBox("Impossible d'crire dans le fichier "
          "de l'historique","Erreur",MB_OK|MB_ICONSTOP);
     iType=HIST_NOHIST;
     close (iFile);
}

int __fastcall THistorique::GetType()
{ /* Faut-il enregistrer l'historique*/
     return iType;
}

bool __fastcall THistorique::Init(int aiPositions[10][10])
{ /* Lit dans le fichier les positions initiales des pions */
  /* aiPositions = Tableau qui recevra les rangs des pios */
  /* Valeur renvoye : true si le fichier a t ouvert et lu avec succs,
     false dans le cas contraire */
     char szHeader[]=ASH_HEADER;
     if((iFile=open(szFileName,O_RDONLY))!=-1) {
          if(read(iFile,&szHeader,sizeof(szHeader))>0) {
               if(!strncmp(szHeader,ASH_HEADER,30)) {
                    if(read(iFile,aiPositions,10*10*sizeof(int))>0) {
                         lseek(iFile,1,SEEK_CUR);
                         return true;
                         /* On a su les lire dans le fichier */
                    }
               }
          }
     }
     return false;
     /* On n'a pas su lire assez */
}

bool __fastcall THistorique::NextMove(TDeplacement *strDepl)
{ /* Lit l'enregistrement suivant dans le fichier d'historique et le place
     dans la structure passe en paramtre. Cre aussi une chane  afficher
     dans la barre d'tat, mais dispose de moins d'informations que la
     fonction AddHistorique() */
  /* strDepl = Structure recevant les champs de l'enregistrement lu */
  /* Valeur renvoye : true si on a lu le dplacement suivant, false si on
     est arriv en fin de fichier */
     if(read(iFile,strDepl,sizeof(TDeplacement))<sizeof(TDeplacement)) {
          return false;
     }
     strcpy(szDepl,"Replay : ");
     char szTemp[256];
     sprintf(szTemp," [%i,%i] ",strDepl->iOld,strDepl->jOld);
     strcat(szDepl,szTemp);
     if(strDepl->iCapture==99)
          sprintf(szTemp,"-> [%i,%i]",strDepl->iNew,strDepl->jNew);
     else if(strDepl->iCapture<0) sprintf(szTemp," est pris par %s [%i,%i]",
          szPiecesHist[(-strDepl->iCapture)%13],strDepl->iNew,strDepl->jNew);
     else if(strDepl->iCapture==0) sprintf(szTemp," et [%i,%i] sont limins",
          strDepl->iNew,strDepl->jNew);
     else sprintf(szTemp," a pris %s [%i,%i]",
               szPiecesHist[strDepl->iCapture%13],strDepl->iNew,strDepl->jNew);
     strcat(szDepl,szTemp); // Chane  afficher
     Affiche(); // Affichage de cette chane
    return true;
}

void __fastcall THistorique::Inverse(int *i1,int *i2,int *i3,int *i4)
{ /* "retourne" les coordonns d'un dplacement, ncessaire si le dplacement
     n'est pas fait avec le plateau orient dans le mme sens que lors du
     replay */
  /* Paramtres : coordonnes  modifier */
     int *i[4]={i1,i2,i3,i4};
     int n;
     for(n=0;n<4;n++) {
          switch(*(i[n])) {
               case 0 : (*(i[n]))=9; break;
               case 1 : (*(i[n]))=8; break;
               case 2 : (*(i[n]))=7; break;
               case 3 : (*(i[n]))=6; break;
               case 4 : (*(i[n]))=5; break;
               case 5 : (*(i[n]))=4; break;
               case 6 : (*(i[n]))=3; break;
               case 7 : (*(i[n]))=2; break;
               case 8 : (*(i[n]))=1; break;
               case 9 : (*(i[n]))=0; break;
          }
     }
}

// FIN <Hist.cpp> ==============================================================

