/************************************************************************
 *                                                                      *
 *   Ant Stratego                                                       *
 *   (C) 2000 Antoine Potten                                            *
 *   software@antp.be - http://www.antp.be/software                     *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 2     *
 *   of the License, or (at your option) any later version.             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   pieces.cpp : Gestion de pions, du plateau et de la partie          *
 *                                                                      *
 ************************************************************************/

#include <vcl.h>
#pragma hdrstop
#include "Pieces.h"
#include "Unit3.h"
#include "Unit5.h"
#include "Unit6.h"
#include "Unit8.h"
/*
     MainWindow -> Partie -> Plateau -> Pion
     MainWindow -> Partie -> Plateau -> ChangeJoueur
     MainWindow -> Partie -> Historique
*/


char szPionFileNames[27][30]={
     // Noms de fichier des images
     "res\\pion_vide.bmp",         // 0
     "res\\pion_rouge_1.bmp",      // 1
     "res\\pion_rouge_2.bmp",      // 2
     "res\\pion_rouge_3.bmp",      // 3
     "res\\pion_rouge_4.bmp",      // 4
     "res\\pion_rouge_5.bmp",      // 5
     "res\\pion_rouge_6.bmp",      // 6
     "res\\pion_rouge_7.bmp",      // 7
     "res\\pion_rouge_8.bmp",      // 8
     "res\\pion_rouge_9.bmp",      // 9
     "res\\pion_rouge_espion.bmp", // 10
     "res\\pion_rouge_bombe.bmp",  // 11
     "res\\pion_rouge_drapeau.bmp",// 12
     "res\\pion_rouge.bmp",        // 13
     "res\\pion_bleu_1.bmp",       // 14
     "res\\pion_bleu_2.bmp",       // 15
     "res\\pion_bleu_3.bmp",       // 16
     "res\\pion_bleu_4.bmp",       // 17
     "res\\pion_bleu_5.bmp",       // 18
     "res\\pion_bleu_6.bmp",       // 19
     "res\\pion_bleu_7.bmp",       // 20
     "res\\pion_bleu_8.bmp",       // 21
     "res\\pion_bleu_9.bmp",       // 22
     "res\\pion_bleu_espion.bmp",  // 23
     "res\\pion_bleu_bombe.bmp",   // 24
     "res\\pion_bleu_drapeau.bmp", // 25
     "res\\pion_bleu.bmp",         // 26
     };
char szPionDescription[27][30]={
     // Infobulles
     "Zone interdite",                  // 0
     "Pion rouge : Marchal",           // 1
     "Pion rouge : Gnral",            // 2
     "Pion rouge : Colonel",            // 3
     "Pion rouge : Commandant",         // 4
     "Pion rouge : Capitaine",          // 5
     "Pion rouge : Lieutenant",         // 6
     "Pion rouge : Sergent",            // 7
     "Pion rouge : Dmineur",           // 8
     "Pion rouge : claireur",          // 9
     "Pion rouge : Espion",             // 10
     "Pion rouge : Bombe",              // 11
     "Pion rouge : Drapeau",            // 12
     "Pion rouge : Unit ennemie",      // 13
     "Pion bleu : Marchal",            // 14
     "Pion bleu : Gnral",             // 15
     "Pion bleu : Colonel",             // 16
     "Pion bleu : Commandant",          // 17
     "Pion bleu : Capitaine",           // 18
     "Pion bleu : Lieutenant",          // 19
     "Pion bleu : Sergent",             // 20
     "Pion bleu : Dmineur",            // 21
     "Pion bleu : claireur",           // 22
     "Pion bleu : Espion",              // 23
     "Pion bleu : Bombe",               // 24
     "Pion bleu : Drapeau",             // 25
     "Pion bleu : Unit ennemie",       // 26
     };
char szErrorMessages[][150]={
     // Messages d'erreur
     "Erreur : il ne devrait pas y avoir de message d'erreur ;-)", //000
     "Erreur : Coordonnes invalides", //001
     "Un dplacement ne peut se faire que dans un seul sens", //002
     "Ce pion ne peut pas tre dplac de plus d'une case", //003
     "Ce pion ne peut pas tre dplac de plus de deux cases", //004
     "Un pion ne peut pas sauter au-dessus d'un autre pion ou au-dessus d'un "
          "lac", //005
     "Ce pion ne peut pas tre dplac", //006
     "Un pion ne peut pas :\n - Sauter un pion\n - Dpasser un pion ennemi ("
          "passer au-dessus d'une case situe  ct d'un pion ennemi)" // 007
     };

// =============================================================================
// 
// TPion =======================================================================

__fastcall TPion::TPion(TComponent* Owner, int iRang,
     int Left, int Top, int iCouleur) : TImage(Owner)
{    /* Constructeur. Appelle d'abord le constructeur de TImage
      : TImage  appelle le constructeur de la classe "parent"
          Left & Top = Position dans le tableau de pions */
     SetCurrentDir(MainWindow->ASDirectory);
     Plateau=Owner;
     Width=PIECE_WIDTH;
     Height=PIECE_HEIGHT;
          /* Position de la pice. Si X > 10, dans la rserve donc il faut
               tenir compte du bord sparant la rserve du plateau de jeu */
     if(Left<10) this->Left=Left*PIECE_WIDTH+PLATEAU_LEFT;
     else this->Left=Left*PIECE_WIDTH+2*PLATEAU_LEFT;
     this->Top=Top*PIECE_HEIGHT+PLATEAU_TOP+TOOLBAR_HEIGHT;
     this->iRang=iRang;
     this->iCouleur=iCouleur;
     if((iCouleur!=PION_VIDE)&&(((TPlateau *)Plateau)->GetMode()!=MODE_REPLAY))
     { /* On permet de dplacer ce pion par drag & drop :
          On ne peut pas bouger les pices de couleur 0 car il
          s'agit des pices invisibles des "lacs", et lors du replay on ne
          peut pas toucher aux pions */
          DragCursor=(TCursor)iRang; // Curseur lors du dplacement
          Cursor=(TCursor)27;// Curseur lors du passage de la souris sur le pion
          DragMode=dmAutomatic; // dmAutomatic = permettre le drag&drop
     }
     Picture->LoadFromFile(szPionFileNames[iRang]);
          // Charger l'image .bmp  partir du disque
     ShowHint=true;
     Hint=szPionDescription[iRang]; /* Copie de la description de la pice
                                   pour la faire apparatre dans l'infobulle */
     Visible=true;
     Transparent=true; /* Transparent = true -> remplace la couleurs n 77 de
                              la palette par du transparent (laisse apparaitre
                              la carte autour de l'image) */
}

void __fastcall TPion::DoEndDrag(TObject* Target, int X, int Y)
{ /* Appel lorsqu'on lche l'image lors du Drag & Drop */
  /* Target = Objet sur lequel le pion a t dpos */
     int iErr;
     Visible=true;
     if(Target==this) return; /* On dpose le pion sur lui-mme (clic sans
          bouger la souris -> on ne fait rien */
     if(Target) {
          /* Si Target != NULL on a dpos le pion sur un objet qui l'a
               accept */
          if(Target->ClassNameIs("TPlateau")) {
               /* Dpos sur le plateau */
               int i, j, iOld, jOld;
                    // Calcul de la case o le pion a t dpos
               if(X<PLATEAU_WIDTH) i=(X-PLATEAU_LEFT)/PIECE_WIDTH;
               else i=(X-2*PLATEAU_LEFT)/PIECE_WIDTH;
               j=(Y-PLATEAU_TOP)/PIECE_HEIGHT;
               iOld=(Left-PLATEAU_LEFT)/PIECE_WIDTH;
               jOld=(Top-PLATEAU_TOP)/PIECE_HEIGHT;
               if((iErr=((TPlateau *)Target)->Deplace(iOld, jOld,
                         i, j, true))==0) {
                         /* Dplacement en mmoire */
                    this->Deplace(i,j); // Dplacement  l'cran
                    if(((TPlateau *)Target)->GetMode()!=MODE_RESERVE) {
                         /* Mise  jour de l'historique, des statistiques,
                              et passage au joueur suivant */
                         bool bJoueurActuel;
                         if(iRang>13) bJoueurActuel=JOUEUR_BLEU;
                         else bJoueurActuel=JOUEUR_ROUGE;
                         MainWindow->Partie->Historique->AddHistorique(
                              bJoueurActuel,iOld,jOld,i,j,iRang,99);
                         StatWindow->AddDeplacement();
                         MainWindow->Partie->JoueurSuivant();
                    }
               } else if(iErr>0) {
                    // Dplacement interdit
                    Application->MessageBox(szErrorMessages[iErr],"Message",
                         MB_OK|MB_ICONINFORMATION);
               }
          } else if(Target->ClassNameIs("TPion")) {
               /* Dpos sur un autre pion -> Combat */
               ((TPlateau *)Plateau)->Combat(
                         (Left-PLATEAU_LEFT)/PIECE_WIDTH,
                         (Top-PLATEAU_TOP)/PIECE_HEIGHT,
                         (((TPion *)Target)->Left-PLATEAU_LEFT)/PIECE_WIDTH,
                         (((TPion *)Target)->Top-PLATEAU_TOP)/PIECE_HEIGHT);
          }
     }
}

void __fastcall TPion::Deplace(int i, int j)
{ /* Dplacement  l'cran d'un pion.
     i, j = coordonnes des cases destination */
     int iLeft, iTop;
     iLeft=PLATEAU_LEFT+i*PIECE_WIDTH;
     iTop=PLATEAU_TOP+j*PIECE_HEIGHT+TOOLBAR_HEIGHT;
     if(i>=10) {
          /* Si plac dans la rserve, il faut tenir compte du bord entre la
               rserve et le plateau de jeu */
          iLeft+=PLATEAU_LEFT;
     }
     Left=iLeft;
     Top=iTop;
}

void __fastcall TPion::Inverse(bool bJoueurActuel, bool bComposer)
{ /* "Retourne" le pion (change l'image, l'infobulle, de curseur, et
     (ds)activation du drag & drop */
  /* bJoueurActuel = Ce sont les pions de ce joueur qui seront visibles
     bComposer = En mode "composer", on ne retourne pas les pions ; on ne fait
                    que modifier les autorisations de dplacement */
     SetCurrentDir(MainWindow->ASDirectory);
     switch(iCouleur) {
          case PION_ROUGE :
               if(bJoueurActuel==JOUEUR_BLEU) {
                    if(bComposer==false) {
                         Picture->LoadFromFile(szPionFileNames[13]);
                         Hint=szPionDescription[13];
                              // Image de pion retourn
                    }
                    DragMode=dmManual;
                    Cursor=crDefault;
                         // Manual = dplacement interdit
               } else {
                    if(bComposer==false) {
                         Picture->LoadFromFile(szPionFileNames[iRang]);
                         Hint=szPionDescription[iRang];
                              // Image d'origine du pion
                    }
                    DragMode=dmAutomatic;
                    Cursor=(TCursor)27;
                         // Dplacement autoris
               }
               break;
          case PION_BLEU :
               if(bJoueurActuel==JOUEUR_ROUGE) {
                    if(bComposer==false) {
                         Picture->LoadFromFile(szPionFileNames[26]);
                         Hint=szPionDescription[26];
                    }
                    DragMode=dmManual;
                    Cursor=crDefault;
               } else {
                    if(bComposer==false) {
                         Picture->LoadFromFile(szPionFileNames[iRang]);
                         Hint=szPionDescription[iRang];
                    }
                    DragMode=dmAutomatic;
                    Cursor=(TCursor)27;
               }
               break;
     }
}

void __fastcall TPion::DragOver(TObject *Source, int X, int Y,
          TDragState State, bool &Accept)
{ /* On doit dire si oui ou non on accepte l'objet que l'on tente de dposer
     sur le pion (drag & drop) (mettre true ou false dans Accept) */
     if(this==Source) {
          /* On place le pion sur lui-mme, c'est  dire que l'on commence 
               le dplacer */
          Visible=false;
          Accept=true;
          return;
     }
     if(Source) {
          if(Source->ClassNameIs("TPion")) {
               /* C'est un autre pion que l'on place sur ce pion */
               switch(MainWindow->Partie->GetMode())
               {
                    case MODE_JEULOCAL:
                    case MODE_JEURESEAU:
                         if(((TPion *)Source)->iCouleur!=iCouleur&&iCouleur)
                              Accept=true;
                              /* On ne peut placer qu'un pion de couleur
                                   diffrente (combat) */
                         else Accept=false;
                         break;
                    case MODE_RESERVE:
                    case MODE_NOMOVE:
                         Accept=false;
                         break;
               }
          }
     }
}

int __fastcall TPion::GetRang()
{ // Rang de la pice (grade)
     return iRang;
}

int __fastcall TPion::GetCouleur()
{ // Couleur de la pice
     return iCouleur;
}

// =============================================================================
// 
// TChangeJoueur ===============================================================

__fastcall TChangeJoueur::TChangeJoueur(TComponent* Owner, bool bJoueur,
          bool bComp) : TImage(Owner)
{ /* Constructeur */
     SetCurrentDir(MainWindow->ASDirectory);
     Width=PLATEAU_WIDTH-PLATEAU_LEFT;
     Height=PLATEAU_HEIGHT-PLATEAU_TOP;
     Plateau=Owner;
     bComposer=bComp;
     switch(bJoueurActuel=bJoueur) {
          case JOUEUR_ROUGE :
               Picture->LoadFromFile("res\\change_rouge.bmp");
               break;
          case JOUEUR_BLEU :
               Picture->LoadFromFile("res\\change_bleu.bmp");
               break;
     }
     MainWindow->InsertControl(this);
     SendToBack();
}

__fastcall TChangeJoueur::~TChangeJoueur()
{
     MainWindow->RemoveControl(this);
}

void __fastcall TChangeJoueur::Affiche()
{ /* Affichage de l'cran intermdiaire */
     Top=PLATEAU_TOP+TOOLBAR_HEIGHT;
     Left=PLATEAU_LEFT;
     MainWindow->StatusBar1->SetTextBuf("");
//     MainWindow->InsertControl(this);
     BringToFront();
     ((TPlateau*)Plateau)->CachePions();  // Pions invisibles
}

void __fastcall TChangeJoueur::Click(void)
{ /* Lorsqu'on clique sur l'image intermdiaire, il faut la faire disparatre
     et retourner tous les pions (si on le fait plus tt, les pions risquent
     d'apparatre un court instant  l'adversaire) */
     if(bComposer==false) {
//          MainWindow->RemoveControl(this);
          SendToBack();
          ((TPlateau*)Plateau)->RetournePions(bJoueurActuel);
     } else if(((TPlateau*)Plateau)->GetReserve()==false) {
          ((TPlateau*)Plateau)->RetournePions(bJoueurActuel);
     }
     ((TPlateau*)Plateau)->AffichePions();  // Pions visibles
     switch(((TPlateau*)Plateau)->GetReserve()) {
          case true :
               MainWindow->EnableButtons(true,bJoueurActuel,
                    false,true,true,false);
               ((TPlateau*)Plateau)->SetMode(MODE_RESERVE);
               MainWindow->Partie->Go2();
                    /* Si on est en mode "placement des pions", il faut
                         voir si on doit commencer la partie ou laisser
                         le joueur suivant placer ses pices (ce que fait la
                         fonction "Go2()" */
               break;
          case false :
               ((TPlateau*)Plateau)->SetMode(MODE_JEULOCAL);
               MainWindow->EnableSauver(bComposer);
               break;
     }
     MainWindow->Partie->Historique->Affiche();
          /* Affichage dans la barre d'tat du dplacement effectu par
               l'autre joueur */
}

// =============================================================================
// 
// TPlateau ====================================================================

__fastcall TPlateau::TPlateau(TComponent* Owner, bool bComp) : TImage(Owner)
{    /* Constructeur standard : Nouveau jeu local */
     bComposer=bComp;
     SetCurrentDir(MainWindow->ASDirectory);
     Width=PLATEAU_HEIGHT;
     Height=PLATEAU_WIDTH;
     Left=0;
     Top=0+TOOLBAR_HEIGHT;
     Picture->LoadFromFile("res\\map.bmp");
     Visible=true;
     MainWindow->InsertControl(this); // Insre le composant dans la fentre
     int i,j;
     for(j=0;j<10;j++) {
          for(i=0;i<14;i++) {
               Cases[i][j]=NULL;
          }
     }
     for(i=0;i<80;i++) {
          Elimines[i]=NULL;
     }
     iNextElimine=0;
     // Cration des pices invisibles reprsentant les lacs
     Cases[2][4]=new TPion(this,0,2,4,PION_VIDE);
     Cases[3][4]=new TPion(this,0,3,4,PION_VIDE);
     Cases[6][4]=new TPion(this,0,6,4,PION_VIDE);
     Cases[7][4]=new TPion(this,0,7,4,PION_VIDE);
     Cases[2][5]=new TPion(this,0,2,5,PION_VIDE);
     Cases[3][5]=new TPion(this,0,3,5,PION_VIDE);
     Cases[6][5]=new TPion(this,0,6,5,PION_VIDE);
     Cases[7][5]=new TPion(this,0,7,5,PION_VIDE);
     for(j=0;j<10;j++) {
          for(i=0;i<14;i++) {
               if(Cases[i][j]!=NULL) MainWindow->InsertControl(Cases[i][j]);
          }
     }
     // crans intermdiaires, affichs lors du changement de joueur
     ChangeJoueurRouge=new TChangeJoueur(this, JOUEUR_ROUGE, bComposer);
     ChangeJoueurBleu=new TChangeJoueur(this, JOUEUR_BLEU, bComposer);
}

__fastcall TPlateau::TPlateau(TComponent* Owner, char *szFileName,
          bool *bJoueurActuel, bool *bCompParam) : TImage(Owner)
{ /* Contructeur recevant un paramtre en plus : chemin d'accs de la
     partie  charger */
     SetCurrentDir(MainWindow->ASDirectory);
     Width=PLATEAU_HEIGHT;
     Height=PLATEAU_WIDTH;
     Left=0;
     Top=0+TOOLBAR_HEIGHT;
     Picture->LoadFromFile("res\\map.bmp");
     Visible=true;
     MainWindow->InsertControl(this); // Insre le composant dans la fentre
     if(!Load(szFileName,bJoueurActuel,bCompParam)) {
          // Erreur lors du chargement de la partie
          MainWindow->LoadError();
          return;
     }
     bComposer=*bCompParam;
     int i,j;
     for(j=0;j<10;j++) {
          for(i=0;i<14;i++) {
               if(Cases[i][j]!=NULL) MainWindow->InsertControl(Cases[i][j]);
          }
     }
     ChangeJoueurRouge=new TChangeJoueur(this, JOUEUR_ROUGE, bComposer);
     ChangeJoueurBleu=new TChangeJoueur(this, JOUEUR_BLEU, bComposer);
     if(bComposer==false) ChangeCote(*bJoueurActuel);
          // On commence directement le jeu
     else RetournePions(*bJoueurActuel);
}

__fastcall TPlateau::TPlateau(TComponent* Owner, int aiPositions[10][10])
           : TImage(Owner)
{ /* Constructeur recevant un tableau : On va rejouer un historique, ce
     tableau correspond aux positions des pions */
     SetCurrentDir(MainWindow->ASDirectory);
     bComposer=false;
     Width=PLATEAU_HEIGHT;
     Height=PLATEAU_WIDTH;
     Left=0;
     Top=0+TOOLBAR_HEIGHT;
     Picture->LoadFromFile("res\\map.bmp");
     Visible=true;
     iMode=MODE_REPLAY;
     iNextElimine=0;
     MainWindow->InsertControl(this); // Insre le composant dans la fentre
     int i,j,iCouleur;
     for(j=0;j<10;j++) {
          for(i=10;i<14;i++) {
               Cases[i][j]=NULL;
          }
     }
     int x,y;
     for(j=0,y=9;j<10;j++,y--) {
          for(i=0,x=9;i<10;i++,x--) {
               if(aiPositions[x][y]==-1)
                    Cases[i][j]=NULL;
               else {
                    if(aiPositions[x][y]==0) iCouleur=PION_VIDE;
                    else if(aiPositions[x][y]<14) iCouleur=PION_ROUGE;
                    else iCouleur=PION_BLEU;
                    Cases[i][j]=new TPion(this,aiPositions[x][y],i,j,iCouleur);
               }
          }
     }
     for(j=0;j<10;j++) {
          for(i=0;i<14;i++) {
               if(Cases[i][j]!=NULL) MainWindow->InsertControl(Cases[i][j]);
          }
     }
     ChangeJoueurRouge=NULL; // On n'aura pas besoin des crans intermdiaires
     ChangeJoueurBleu=NULL;
}

__fastcall TPlateau::~TPlateau()
{ // Destructeur
     int i,j;
     for(j=0;j<10;j++) {
          for(i=0;i<14;i++) {
               if(Cases[i][j]!=NULL) {
                    // Destruction des tous les pions
                    MainWindow->RemoveControl(Cases[i][j]);
                    delete (Cases[i][j]);
                    Cases[i][j]=NULL;
               }
          }
     }
     for(i=0;i<iNextElimine;i++) {
          if(Elimines[i]!=NULL) {
               delete (Elimines[i]);
               Elimines[i]=NULL;
          }
     }
     if(ChangeJoueurRouge!=NULL ) {
          delete ChangeJoueurRouge;
          ChangeJoueurRouge=NULL;
     }
     if(ChangeJoueurBleu !=NULL ) {
          delete ChangeJoueurBleu;
          ChangeJoueurBleu=NULL;
     }
}

bool __fastcall TPlateau::GetReserve()
{ // Rserve active ou non (c--d mode "placement des pions")
     return bReserve;
}

void __fastcall TPlateau::CachePions()
{ // Cache tous les pions du plateau
     int i,j;
     for(j=0;j<10;j++) {
          for(i=0;i<10;i++) {
               if(Cases[i][j]!=NULL) {
                    Cases[i][j]->Visible=false;
               }
          }
     }
}

void __fastcall TPlateau::AffichePions()
{ // Raffiche tous les pions
     int i,j;
     for(j=0;j<10;j++) {
          for(i=0;i<10;i++) {
               if(Cases[i][j]!=NULL) {
                    Cases[i][j]->Visible=true;
               }
          }
     }
}

int __fastcall TPlateau::Deplace(int iSource, int jSource,
          int iTarget, int jTarget, bool bVerifCases)
{ /* Dplacement du pion en mmoire (et test de la validit du dplacement
     si on est en mode normal (pas en combat) -> selon bVerifCases) */
  /* i & j : Coordonnes de la case source et de la case destination */
     if(bVerifCases) {
          if(Cases[iSource][jSource]==Cases[iTarget][jTarget]) return -1;
               /* On place le pion sur la case o il tait avant -> il ne
                    faudra pas considrer cela comme un dplacement */
          if(Cases[iTarget][jTarget]!=NULL) return 001;
               /* En dplacement normal, la case destination ne peut pas
                    tre occupe */
     }
     if(Cases[iSource][jSource]==NULL) return 001;
          // Pas de pion dans la case source
     if(   iSource>=14||jSource>=10||iTarget>=14||jTarget>=10
         ||iSource< 0 ||jSource< 0 ||iTarget< 0 ||jTarget< 0  )
          return 001;
          // Dbordement du tableau
     if(iMode==MODE_JEULOCAL||iMode==MODE_JEURESEAU) {
          /* Si on n'est pas en mode "placement des pions", il y a des rgles
                suivre */
          if(iSource!=iTarget) if(jSource!=jTarget) return 002;
          if(jSource!=jTarget) if(iSource!=iTarget) return 002;
               // On ne peut bouger que dans un sens
          switch(Cases[iSource][jSource]->GetRang()) {
               case 0: case 13: case 26:
               case 11: case 12:
               case 24: case 25:
                    return 006; /* On ne peut pas bouger les pions retourns,
                         les lacs, les drapeaux et les bombes */
               case 9:
               case 22:
                    /* claireurs : ils peuvent bouger d'autant de cases qu'ils
                         le veulent mais :
                              - Ils ne peuvent pas sauter au-dessus d'une autre
                                   pice
                              - Ils ne peuvent pas passer au-dessus d'une case
                                   voisine d'un pion ennemi
                    */
                    int n;
                    for(n=min(jSource,jTarget)+1;n<max(jSource,jTarget);n++) {
                         if(Cases[iSource][n]!=NULL) return 005;
                         if(iSource>0) {
                           if(Cases[iSource-1][n]!=NULL)
                             if(Cases[iSource-1][n]->GetRang()>0
                                   &&Cases[iSource-1][n]->GetCouleur()
                                        !=Cases[iSource][jSource]->GetCouleur())
                               return 007;
                         }
                         if(iSource<9) {
                           if(Cases[iSource+1][n]!=NULL)
                             if(Cases[iSource+1][n]->GetRang()>0
                                   &&Cases[iSource+1][n]->GetCouleur()
                                        !=Cases[iSource][jSource]->GetCouleur())
                               return 007;
                         }
                    }
                    for(n=min(iSource,iTarget)+1;n<max(iSource,iTarget);n++) {
                         if(Cases[n][jSource]!=NULL) return 005;
                         if(jSource>0) {
                           if(Cases[n][jSource-1]!=NULL)
                             if(Cases[n][jSource-1]->GetRang()>0
                                   &&Cases[n][jSource-1]->GetCouleur()
                                        !=Cases[iSource][jSource]->GetCouleur())
                               return 007;
                         }
                         if(jSource<9) {
                           if(Cases[n][jSource+1]!=NULL)
                             if(Cases[n][jSource+1]->GetRang()>0
                                   &&Cases[n][jSource+1]->GetCouleur()
                                        !=Cases[iSource][jSource]->GetCouleur())
                               return 007;
                         }
                    }
                    break;
               default :
                    // Pour les autres pions, c'est maximum une case
                    if(abs(jSource-jTarget)>1) return 003;
                    if(abs(iSource-iTarget)>1) return 003;
                    break;
          }
     }
     if(bVerifCases) {
          Cases[iTarget][jTarget]=Cases[iSource][jSource];
          Cases[iSource][jSource]=NULL;
               /* Dplacement du pion dans le tableau de pointeurs. En mode
                    dplacement normal, cette fonction se charge aussi de
                    modifier les pointeurs */
     }
     return 000; // Pas d'erreurs
}

void __fastcall TPlateau::DragOver(TObject *Source, int X, int Y,
          TDragState State, bool &Accept)
{    /* Accept : si vrai le composant accepte l'objet qu'on est occup 
          glisser dessus */
     if(Source) {
       if(Source->ClassNameIs("TPion")) {
          switch(iMode) {
            case MODE_JEULOCAL :  // Pendant le jeu normal
            case MODE_JEURESEAU :
               if(( X>=PLATEAU_LEFT && X<=PLATEAU_WIDTH-PLATEAU_LEFT )
                         && ( Y>=PLATEAU_TOP && Y<=PLATEAU_HEIGHT-PLATEAU_TOP ))
               {
                    // On ne peut placer les pices que dans le plateau
                    Accept=true;
               } else { Accept=false; }
               break;
            case MODE_RESERVE : // Pendant le placement des pions
               if((( X>=PLATEAU_LEFT && X<=PLATEAU_WIDTH-PLATEAU_LEFT )
                         || ( X>PLATEAU_WIDTH+1
                                   && X<PLATEAU_RESERVE_WIDTH-PLATEAU_LEFT))
                         && ( Y>=PLATEAU_TOP && Y<=PLATEAU_HEIGHT-PLATEAU_TOP ))
               { /* On peut placer les pices dans le plateau ou sur dans
                    la rserve */
                    if(bComposer==false
                              &&Y<PLATEAU_TOP+6*PIECE_HEIGHT&&X<PLATEAU_WIDTH) {
                         /* Si on n'est pas en mode "composer", on ne peut
                              placer les pions que dans les 4 ranges du bas */
                         Accept=false;
                    } else Accept=true;
               } else { Accept=false; }
               break;
            case MODE_NOMOVE :
               Accept=false;
               break;
          }
       } else Accept=false;
     } else Accept=false;
}

void __fastcall TPlateau::SetMode(int iNewMode)
{ // Change le mode actuel
     iMode=iNewMode;
}

int __fastcall TPlateau::GetMode()
{ // Retourne le mode actuel
     return iMode;
}

void __fastcall TPlateau::EnableReserve(bool bState)
{ // Affiche ou cache la rserve situe  droite de la carte
     SetCurrentDir(MainWindow->ASDirectory);
     if(bReserve!=bState) {
          /* Si == alors pas besoin de rafficher la rserve ou de la cacher,
               elle est dj bien comme elle est */
          bReserve=bState;
          switch(bState) {
               case true :
                    // Redimmensionnement de la fentre
                    MainWindow->ClientWidth=PLATEAU_RESERVE_WIDTH;
                    MainWindow->ClientHeight=PLATEAU_HEIGHT
                         +TOOLBAR_HEIGHT+STATUSBAR_HEIGHT+BUTTONS_HEIGHT;
                    // Modification de l'image
                    Width=PLATEAU_RESERVE_WIDTH;
                    Picture->LoadFromFile("res\\map_reserve.bmp");
                    MainWindow->EnableButtons(true,JOUEUR_ROUGE,
                                                       true,false,false,false);
                    break;
               case false :
                    Width=PLATEAU_WIDTH;
                    MainWindow->ClientWidth=PLATEAU_WIDTH;
                    MainWindow->ClientHeight=PLATEAU_HEIGHT
                         +TOOLBAR_HEIGHT+STATUSBAR_HEIGHT;
                    MainWindow->EnableButtons(false,false,false,false,
                                                  false,false);
                    Picture->LoadFromFile("res\\map.bmp");
                    break;;
          }
     } else {
          /* Si on ne doit pas recharger le .bmp, il est quand mme
               plus prudent de redimensionner la fentre ... */
          MainWindow->ClientWidth=Width;
          MainWindow->ClientHeight=PLATEAU_HEIGHT+TOOLBAR_HEIGHT
               +STATUSBAR_HEIGHT;
     }
     if(MainWindow->GetRecentrer()) {
          // ... Et ventuellement de la recentrer
          MainWindow->Position=poScreenCenter;
     }
}

bool __fastcall TPlateau::ValiderPieces()
{    /* Vrifie que toutes les pices ont t places sur la carte */
     /* Retourne faux si des pices restent dans la rserve, vrai si la
          rserve est vide */
     /* En mode "composer", ne fait que vrifier que le drapeau a dj t
          plac */
     int i,j;
     for(j=0;j<10;j++) {
          for(i=10;i<14;i++) {
               switch(bComposer) {
                    case true :
                         if(Cases[i][j]!=NULL)
                              if(Cases[i][j]->GetRang()==12
                                   ||Cases[i][j]->GetRang()==25) return false;
                         break;
                    case false :
                         if(Cases[i][j]!=NULL) return false;
                         break;
               }
          }
     }
     return true;
}

bool __fastcall TPlateau::PlacerReserve(bool bCouleur)
{ /* Place les pions de couleur "bCouleur" dans la rserve */
  /* Renvoie faux dans le cas o la rserve n'tait pas vide */
     int i,j;
     int iIndiceCouleur, iCouleurPion;
     for(j=0;j<10;j++) {
          for(i=10;i<14;i++) {
               if(Cases[i][j]!=NULL) return false;
          }
     }
     switch(bCouleur) {
          case JOUEUR_ROUGE :
               iIndiceCouleur=0;
               iCouleurPion=PION_ROUGE;
               break;
          case JOUEUR_BLEU :
               iIndiceCouleur=13;
               iCouleurPion=PION_BLEU;
               break;
     }
     Cases[10][0]=new TPion(this,iIndiceCouleur+10,10,0,iCouleurPion);
     Cases[11][0]=new TPion(this,iIndiceCouleur+1,11,0,iCouleurPion);
     Cases[12][0]=new TPion(this,iIndiceCouleur+2,12,0,iCouleurPion);
     Cases[13][0]=new TPion(this,iIndiceCouleur+3,13,0,iCouleurPion);
     Cases[10][1]=new TPion(this,iIndiceCouleur+3,10,1,iCouleurPion);
     Cases[11][1]=new TPion(this,iIndiceCouleur+4,11,1,iCouleurPion);
     Cases[12][1]=new TPion(this,iIndiceCouleur+4,12,1,iCouleurPion);
     Cases[13][1]=new TPion(this,iIndiceCouleur+4,13,1,iCouleurPion);
     Cases[10][2]=new TPion(this,iIndiceCouleur+5,10,2,iCouleurPion);
     Cases[11][2]=new TPion(this,iIndiceCouleur+5,11,2,iCouleurPion);
     Cases[12][2]=new TPion(this,iIndiceCouleur+5,12,2,iCouleurPion);
     Cases[13][2]=new TPion(this,iIndiceCouleur+5,13,2,iCouleurPion);
     Cases[10][3]=new TPion(this,iIndiceCouleur+6,10,3,iCouleurPion);
     Cases[11][3]=new TPion(this,iIndiceCouleur+6,11,3,iCouleurPion);
     Cases[12][3]=new TPion(this,iIndiceCouleur+6,12,3,iCouleurPion);
     Cases[13][3]=new TPion(this,iIndiceCouleur+6,13,3,iCouleurPion);
     Cases[10][4]=new TPion(this,iIndiceCouleur+7,10,4,iCouleurPion);
     Cases[11][4]=new TPion(this,iIndiceCouleur+7,11,4,iCouleurPion);
     Cases[12][4]=new TPion(this,iIndiceCouleur+7,12,4,iCouleurPion);
     Cases[13][4]=new TPion(this,iIndiceCouleur+7,13,4,iCouleurPion);
     Cases[10][5]=new TPion(this,iIndiceCouleur+8,10,5,iCouleurPion);
     Cases[11][5]=new TPion(this,iIndiceCouleur+8,11,5,iCouleurPion);
     Cases[12][5]=new TPion(this,iIndiceCouleur+8,12,5,iCouleurPion);
     Cases[13][5]=new TPion(this,iIndiceCouleur+8,13,5,iCouleurPion);
     Cases[10][6]=new TPion(this,iIndiceCouleur+8,10,6,iCouleurPion);
     Cases[11][6]=new TPion(this,iIndiceCouleur+9,11,6,iCouleurPion);
     Cases[12][6]=new TPion(this,iIndiceCouleur+9,12,6,iCouleurPion);
     Cases[13][6]=new TPion(this,iIndiceCouleur+9,13,6,iCouleurPion);
     Cases[10][7]=new TPion(this,iIndiceCouleur+9,10,7,iCouleurPion);
     Cases[11][7]=new TPion(this,iIndiceCouleur+9,11,7,iCouleurPion);
     Cases[12][7]=new TPion(this,iIndiceCouleur+9,12,7,iCouleurPion);
     Cases[13][7]=new TPion(this,iIndiceCouleur+9,13,7,iCouleurPion);
     Cases[10][8]=new TPion(this,iIndiceCouleur+9,10,8,iCouleurPion);
     Cases[11][8]=new TPion(this,iIndiceCouleur+12,11,8,iCouleurPion);
     Cases[12][8]=new TPion(this,iIndiceCouleur+11,12,8,iCouleurPion);
     Cases[13][8]=new TPion(this,iIndiceCouleur+11,13,8,iCouleurPion);
     Cases[10][9]=new TPion(this,iIndiceCouleur+11,10,9,iCouleurPion);
     Cases[11][9]=new TPion(this,iIndiceCouleur+11,11,9,iCouleurPion);
     Cases[12][9]=new TPion(this,iIndiceCouleur+11,12,9,iCouleurPion);
     Cases[13][9]=new TPion(this,iIndiceCouleur+11,13,9,iCouleurPion);
     for(j=0;j<10;j++) {
          for(i=10;i<14;i++) {
               if(Cases[i][j]!=NULL)
                    MainWindow->InsertControl(Cases[i][j]);
          }
     }
     return true;
}

void __fastcall TPlateau::PlacerPiecesAuto(bool bJoueur)
{ /* Fonction plaant automatiquement les pices sur le plateau */
  /* En mode "composer", les pices sont places n'importe o sur le plateau ;
     sinon c'est seulement dans la partie infrieure */
     int iR, jR, iP, jP, jPStart;
     bool bFound;
     if(bComposer==true) jPStart=0;
     else jPStart=6;
     for(iR=10;iR<14;iR++) {
          for(jR=0;jR<10;jR++) {
               if(Cases[iR][jR]!=NULL) {
                    randomize();
                    for(bFound=false;bFound==false;) {
                         iP=rand()%(10);
                         jP=rand()%(10-jPStart);
                         jP+=jPStart;
                         if(Cases[iP][jP]==NULL) bFound=true;
                    }
                    Deplace(iR,jR,iP,jP,true);
                    Cases[iP][jP]->Deplace(iP,jP);
               }
          }
     }

}

void __fastcall TPlateau::ChangeCote(bool bJoueurActuel)
{    /* Affiche le .bmp qui demande le changement de joueur */
     if(bReserve==false) MainWindow->EnableSauver(true);
          /* On ne peut sauver qu'entre deux dplacements, pendant que cette
               image intermdiaire est affiche */
     switch(bJoueurActuel) {
          case JOUEUR_ROUGE :
               ChangeJoueurRouge->Affiche();
               SetMode(MODE_NOMOVE);
               break;
          case JOUEUR_BLEU :
               ChangeJoueurBleu->Affiche();
               SetMode(MODE_NOMOVE);
               break;
     }
}

void __fastcall TPlateau::RetournePions(bool bJoueurActuel)
{    /* change les pices de ct */
     int iT,jT,iD,jD,i,j;
     if(bComposer==false) {
          TPion *CaseSwap;
          for(jT=0,jD=9;jT<5&&jD>4;jT++,jD--) {
               for(iT=0,iD=9;iT<10&&iD>=0;iT++,iD--) {
               /* Inversion dans tableau en mmoire */
                    CaseSwap=Cases[iT][jT];
                    Cases[iT][jT]=Cases[iD][jD];
                    Cases[iD][jD]=CaseSwap;
                    if(Cases[iT][jT]!=NULL) Cases[iT][jT]->Deplace(iT,jT);
                    if(Cases[iD][jD]!=NULL) Cases[iD][jD]->Deplace(iD,jD);
               }
          }
     }
     /* Inversion "physique" ( l'cran) des pions */
     for(j=0;j<10;j++) {
          for(i=0;i<10;i++) {
               if(Cases[i][j]!=NULL)
                    Cases[i][j]->Inverse(bJoueurActuel,bComposer);
          }
     }
}

void __fastcall TPlateau::ComptePieces(int aiPieces[6][12])
{ /* Appele par la fentre des statistiques, cette fonction compte les
     pions et classes ces comptes dans un tableau (pass en paramtre) */
     int i,j,iRang;
     for(j=0;j<10;j++) {
          /* Pions de la carte */
          for(i=0;i<10;i++) {
               if(Cases[i][j]!=NULL) {
                    iRang=Cases[i][j]->GetRang();
                    if(iRang>0) {
                         if(iRang<13) {
                              aiPieces[0][iRang-1]++;
                              aiPieces[1][iRang-1]++;
                         } else if(iRang>13&&iRang<26) {
                              aiPieces[0][iRang-14]++;
                              aiPieces[2][iRang-14]++;
                         }
                    }
               }
          }
     }
     for(i=0;i<iNextElimine;i++) {
          /* Pions limins */
          if(Elimines[i]!=NULL) {
               iRang=Elimines[i]->GetRang();
               if(iRang>0) {
                    if(iRang<13) {
                         aiPieces[3][iRang-1]++;
                         aiPieces[4][iRang-1]++;
                    } else if(iRang>13&&iRang<26) {
                         aiPieces[3][iRang-14]++;
                         aiPieces[5][iRang-14]++;
                    }
               }
          }
     }
}

void __fastcall TPlateau::Elimine(int i,int j)
{ /* Supprime un pion de la carte et le place dans le tableau des pions
     limins */
     if(Cases[i][j]!=NULL) {
          MainWindow->RemoveControl(Cases[i][j]);
          Elimines[iNextElimine++]=Cases[i][j];
          Cases[i][j]=NULL;
     }
}

void __fastcall TPlateau::Combat(int iAttaqu, int jAttaqu,
                                   int iVictime, int jVictime)
{ /* Combat de pions : compare les rangs des pions, gre le dplacement,
     la suppression, l'ajout  l'historique, aux statistiques, la
     fin de partie ventuelle et l'affichage de la bote de dialogue */
  /* En paramtres : les coordonnes du pion que l'on a dplac (attaquant)
     et de la case o on l'a dpos (pion attaqu -> victime) */
     int iRangA,iRangV,iVainqueur;
     int iErr;
     if((iErr=Deplace(iAttaqu, jAttaqu, iVictime, jVictime, false))!=0) {
          // Dplacement non valide
          Application->MessageBox(szErrorMessages[iErr],"Message",
               MB_OK|MB_ICONINFORMATION);
          return;
     }
     bool bJoueurActuel;
     switch(Cases[iAttaqu][jAttaqu]->GetCouleur()) {
          case PION_ROUGE : bJoueurActuel = JOUEUR_ROUGE; break;
          case PION_BLEU  : bJoueurActuel = JOUEUR_BLEU;  break;
     }
     switch(iVainqueur=Vainqueur(iRangA=Cases[iAttaqu][jAttaqu]->GetRang(),
                         iRangV=Cases[iVictime][jVictime]->GetRang()))
     { // Dtermine quel est le rsultat du combat
          case  1 : // Attaquant a gagn
               MainWindow->Partie->Historique->AddHistorique(bJoueurActuel,
                         iAttaqu,jAttaqu,iVictime,jVictime,
                         Cases[iAttaqu][jAttaqu]->GetRang(),
                         Cases[iVictime][jVictime]->GetRang());
               Elimine(iVictime,jVictime);
               Cases[iVictime][jVictime]=Cases[iAttaqu][jAttaqu];
               Cases[iAttaqu][jAttaqu]=NULL;
               Cases[iVictime][jVictime]->Deplace(iVictime,jVictime);
                    // Dplacement "physique" ( l'cran)
               StatWindow->AddVictoire(Cases[iVictime][jVictime]->GetCouleur());
               break;
          case -1 : // Attaquant a perdu
               MainWindow->Partie->Historique->AddHistorique(bJoueurActuel,
                         iAttaqu,jAttaqu,iVictime,jVictime,
                         Cases[iAttaqu][jAttaqu]->GetRang(),
                         -(Cases[iVictime][jVictime]->GetRang()));
               Elimine(iAttaqu,jAttaqu);
               StatWindow->AddVictoire(Cases[iVictime][jVictime]->GetCouleur());
               break;
          case  0 : // Attaquant et attaqu ont perdu
               MainWindow->Partie->Historique->AddHistorique(bJoueurActuel,
                         iAttaqu,jAttaqu,iVictime,jVictime,
                         Cases[iAttaqu][jAttaqu]->GetRang(),0);
               Elimine(iVictime,jVictime);
               Elimine(iAttaqu,jAttaqu);
               break;
          case 12:
          case 25: // Le drapeau a t dcouvert
               MainWindow->Partie->Historique->AddHistorique(bJoueurActuel,
                         iAttaqu,jAttaqu,iVictime,jVictime,
                         Cases[iAttaqu][jAttaqu]->GetRang(),
                         Cases[iVictime][jVictime]->GetRang());
               Elimine(iVictime,jVictime);
               Cases[iVictime][jVictime]=Cases[iAttaqu][jAttaqu];
               Cases[iAttaqu][jAttaqu]=NULL;
               Cases[iVictime][jVictime]->Deplace(iVictime,jVictime);
                    // Dplacement "physique" ( l'cran)
               StatWindow->AddVictoire(Cases[iVictime][jVictime]->GetCouleur());
               SetMode(MODE_NOMOVE);
               break;
     }
     StatWindow->AddDeplacement();
          // Dtermination du vainqueur
     if(iVainqueur==-1) iVainqueur=iRangV;
     else if(iVainqueur==1) iVainqueur=iRangA;
     else if(iVainqueur==0) iVainqueur=-1;
     else {
          // Partie termine : le drapeau a t trouv
          MainWindow->EnableSauver(false);
          VainqueurWindow->Victoire(iRangA,iRangV);
          VainqueurWindow->ShowModal();
          return;
     }
     /* Sinon affichage du rsultat du combat, et passage au joueur suivant
          lorsque l'on ferme la boite de dialogue */
     CombatWindow->Combat(iRangA,iRangV,iVainqueur);
     switch(CombatWindow->ShowModal()) {
          default : MainWindow->Partie->JoueurSuivant();
     }
}

int __fastcall TPlateau::Vainqueur(int iRangA, int iRangV)
{
     if(iRangV==12||iRangV==25) return iRangV; // drapeau trouv
     if(iRangA>13) iRangA-=13;         // mettre au mme niveau
     else if(iRangV>13) iRangV-=13;    //
     if(iRangA==iRangV) return 0; // mmes pices
     if(iRangA==10&&iRangV==1) return 1; // espion sur marchal
     if(iRangA==8 &&iRangV==11) return 1; // dmineur sur bombe
     if(iRangV==11) return -1; // autre sur bombe
     if(iRangA>iRangV) return -1;
     if(iRangA<iRangV) return 1;
     return 0; // normalement impossible
}

bool __fastcall TPlateau::Save(char *szFileName, bool bAffichePrises,
     bool bJoueurActuel)
{ /* Sauvegarde dans un fichier des infos ncessaires pour recrer une partie */
  /* szFileName = Nom du fichier dans lequel il faut sauver
     bAfficherPrises = non utilis pour le moment
     bJoueurActuel = Joueur actuel au moment de la sauvegarde */
  /* Retourne vrai ou faux selon que la sauvegarde a su tre effectue ou non */
     int f=open(szFileName,O_WRONLY|O_CREAT|O_TRUNC,S_IREAD|S_IWRITE);
     if(f==-1) return false;
     memset((void *)&strSavedGame,0,sizeof(strSavedGame));
     strncpy(strSavedGame.szID,ASG_HEADER,30);
     strncpy(strSavedGame.szEOF,ASG_EOFILE,10);
     strSavedGame.iNextElimine=iNextElimine;
     strSavedGame.bAffichePrises=bAffichePrises;
     strSavedGame.bJoueurActuel=bJoueurActuel;
     strSavedGame.bComposer=bComposer;
     strSavedGame.iDeplacements=StatWindow->GetDeplacements();
     strSavedGame.iVictoiresRouges=StatWindow->GetVictoires(PION_ROUGE);
     strSavedGame.iVictoiresBleues=StatWindow->GetVictoires(PION_BLEU);
     int i,j;
     for(j=0;j<10;j++) {
          for(i=0;i<10;i++) {
               if(Cases[i][j]==NULL)
                    strSavedGame.aiPions[i][j]=-1;
               else strSavedGame.aiPions[i][j]=Cases[i][j]->GetRang();
          }
     }
     for(i=0;i<iNextElimine;i++) {
          strSavedGame.aiElimines[i]=Elimines[i]->GetRang();
     }
     int iWrite=write(f,(void *)&strSavedGame,sizeof(strSavedGame));
     close(f);
     return (iWrite!=-1);
}

bool __fastcall TPlateau::Load(char *szFileName, bool *bJoueurActuel,
     bool *bCompParam)
{ /* Lecture dans un fichier et mise en place de ce qui a t lu */
  /* szFileName = Nom du fichier
     bJoueurActuel = Pointeur permettant de donner  "Partie" le joueur actuel
     Retourne vrai ou faux selon que le chargement a su tre effectu ou non */
     int f=open(szFileName,O_RDONLY);
     if(f==-1) return false;
     TSavedGameEnreg strSavedGame;
     int iRead=read(f,(void *)&strSavedGame,sizeof(strSavedGame));
     if(iRead!=sizeof(strSavedGame)-1) return false;
     if(strncmp(strSavedGame.szID,ASG_HEADER,30)) return false;
     if(strncmp(strSavedGame.szEOF,ASG_EOFILE,10)) return false;
     iNextElimine=strSavedGame.iNextElimine;
     *bJoueurActuel=strSavedGame.bJoueurActuel;
     *bCompParam=strSavedGame.bComposer;
     bComposer=strSavedGame.bComposer;
     StatWindow->SetDeplacements(strSavedGame.iDeplacements);
     StatWindow->SetVictoires(strSavedGame.iVictoiresRouges,
                              strSavedGame.iVictoiresBleues);
     int i,j;
     for(j=0;j<10;j++) {
          for(i=10;i<14;i++) {
               Cases[i][j]=NULL;
          }
     }
     int iCouleur;
     for(j=0;j<10;j++) {
          for(i=0;i<10;i++) {
               if(strSavedGame.aiPions[i][j]==-1)
                    Cases[i][j]=NULL;
               else {
                    if(strSavedGame.aiPions[i][j]==0) iCouleur=PION_VIDE;
                    else if(strSavedGame.aiPions[i][j]<14) iCouleur=PION_ROUGE;
                    else iCouleur=PION_BLEU;
                    Cases[i][j]=new TPion(this,strSavedGame.aiPions[i][j],
                         i,j,iCouleur);
               }
          }
     }
     for(i=0;i<iNextElimine;i++) {
          if(strSavedGame.aiElimines[i]<14) iCouleur=PION_ROUGE;
          else iCouleur=PION_BLEU;
          Elimines[i]=new TPion(this,strSavedGame.aiElimines[i],0,0,iCouleur);
     }
     return true;
}

void __fastcall TPlateau::EnregHistorique(int aiPositions[10][10],
          bool bInverse)
{ /* Enregistrement des positions initiales des pions pour l'historique. Les
     rangs des pions sont inscrits dans un tableau */
  /* Si inverse = vrai, alors il s'agit d'un cas particulier : une partie
     charge o c'est au joueur rouge de joueur. Dans ce cas il ne faudra
     pas retourner le tableau (voir plus bas) */
     int i,j,x,y;
     for(j=0,y=9;j<10;j++,y--) {
          for(i=0,x=9;i<10;i++,x--) {
     /* x et y car le tableau que l'ont crit dans le fichier doit tre invers
          par rapport  celui qui est en mmoire */
               if(bInverse==false) {
                    if(Cases[i][j]==NULL)
                         aiPositions[x][y]=-1;
                    else aiPositions[x][y]=Cases[i][j]->GetRang();
               } else {
                    if(Cases[i][j]==NULL)
                         aiPositions[i][j]=-1;
                    else aiPositions[i][j]=Cases[i][j]->GetRang();
               }
          }
     }
}

void __fastcall TPlateau::NextMove(TDeplacement strDepl)
{ /* Effectue le dplacement suivant lors du replay, selon les informations
     trouves dans la structure passe en paramtre */
     if(strDepl.iCapture<0) {
          // Pion limin
          MainWindow->RemoveControl(Cases[strDepl.iOld][strDepl.jOld]);
          delete Cases[strDepl.iOld][strDepl.jOld];
          Cases[strDepl.iOld][strDepl.jOld]=NULL;
     } else if(strDepl.iCapture>0&&strDepl.iCapture!=99) {
          // Pion en liminant un autre
          MainWindow->RemoveControl(Cases[strDepl.iNew][strDepl.jNew]);
          delete Cases[strDepl.iNew][strDepl.jNew];
          Cases[strDepl.iNew][strDepl.jNew]=NULL;
          Cases[strDepl.iNew][strDepl.jNew]=Cases[strDepl.iOld][strDepl.jOld];
          Cases[strDepl.iOld][strDepl.jOld]=NULL;
          Cases[strDepl.iNew][strDepl.jNew]->Deplace(strDepl.iNew,strDepl.jNew);
     } else if(strDepl.iCapture==0) {
          // Deux pions limins
          MainWindow->RemoveControl(Cases[strDepl.iOld][strDepl.jOld]);
          delete Cases[strDepl.iOld][strDepl.jOld];
          Cases[strDepl.iOld][strDepl.jOld]=NULL;
          MainWindow->RemoveControl(Cases[strDepl.iNew][strDepl.jNew]);
          delete Cases[strDepl.iNew][strDepl.jNew];
          Cases[strDepl.iNew][strDepl.jNew]=NULL;
     } else {
          // Dplacement normal
          Cases[strDepl.iNew][strDepl.jNew]=Cases[strDepl.iOld][strDepl.jOld];
          Cases[strDepl.iOld][strDepl.jOld]=NULL;
          Cases[strDepl.iNew][strDepl.jNew]->Deplace(strDepl.iNew,strDepl.jNew);
     }
}

void __fastcall TPlateau::RemoveChangeJoueur(bool bJoueur)
{ // Fait disparatre l'cran intermdiaire sans que l'on doive cliquer dessus
     switch(bJoueur) {
          case JOUEUR_ROUGE :
               ChangeJoueurRouge->Click();
               break;
          case JOUEUR_BLEU  :
               ChangeJoueurBleu->Click();
               break;
     }
}

void __fastcall TPlateau::VideReserve()
{ // Supprime tous les pions restant dans la rserve
     int i,j;
     for(j=0;j<10;j++) {
          for(i=10;i<14;i++) {
               if(Cases[i][j]!=NULL) {
                    MainWindow->RemoveControl(Cases[i][j]);
                    delete Cases[i][j];
                    Cases[i][j]=NULL;
               }
          }
     }
}

// =============================================================================
// 
// TPartie =====================================================================

__fastcall TPartie::TPartie()
{ /* Constructeur pour une partie normale (jeu local) */
     bComposer=false;
     MainWindow->StatusBar1->SetTextBuf("");
     bRougePlaces=false;
     bBleuPlaces=false;
     MainWindow->EnableReplayControl(false,false,false,false);
     Plateau=new TPlateau(MainWindow,false);
     Plateau->EnableReserve(true);
     char *szReplayFile=NULL;
     int iReplayType=MainWindow->GetReplayStyle(&szReplayFile);
     Historique = new THistorique(iReplayType,szReplayFile,bComposer);
     delete szReplayFile;
     bJoueurActuel=JOUEUR_BLEU;
     JoueurSuivant();
}

__fastcall TPartie::TPartie(bool bParam)
{ /* Constructeur pour une partie en mode "composer". Le paramtre ne sert 
     rien, sauf  distinguer les deux constructeurs */
     bComposer=true;
     MainWindow->StatusBar1->SetTextBuf("");
     bRougePlaces=false;
     bBleuPlaces=false;
     MainWindow->EnableReplayControl(false,false,false,false);
     Plateau=new TPlateau(MainWindow,true);
     Plateau->EnableReserve(true);
     char *szReplayFile=NULL;
     int iReplayType=MainWindow->GetReplayStyle(&szReplayFile);
     Historique = new THistorique(iReplayType,szReplayFile,bComposer);
     delete szReplayFile;
     bJoueurActuel=JOUEUR_BLEU;
     JoueurSuivant();
}

__fastcall TPartie::TPartie(char *szFileName)
{ /* Constructeur effectuant le chargement d'une partie  partir du fichier
     szFileName */
     bComposer=false;
     MainWindow->StatusBar1->SetTextBuf("");
     MainWindow->EnableReplayControl(false,false,false,false);
     MainWindow->EnableButtons(false,false,false,false,false,false);
     bRougePlaces=true;
     bBleuPlaces=true;
     Plateau=new TPlateau(MainWindow, szFileName, &bJoueurActuel, &bComposer);
     Plateau->EnableReserve(false);
     MainWindow->EnableStats(true);
     MainWindow->EnableSauver(bComposer);
     char *szReplayFile=NULL;
     int iReplayType=MainWindow->GetReplayStyle(&szReplayFile);
     Historique = new THistorique(iReplayType,szReplayFile,bComposer);
     delete szReplayFile;
     SetMode(MODE_JEULOCAL);
     EnregHistorique(bJoueurActuel);
}

__fastcall TPartie::TPartie(int iSpeed, char *szFileName)
{ /* Constructeur pour une partie servant  rejoueur un historique */
  /* szFileName = nom du fichier, iSpeed = vitesse du replay (ne sert en fait
      rien, mais ici encore permet de diffrencier les constructeurs) */
     bComposer=false;
     MainWindow->StatusBar1->SetTextBuf("");
     MainWindow->EnableStats(false);
     MainWindow->EnableSauver(false);
     MainWindow->EnableButtons(false,false,false,false,false,false);
     Historique = new THistorique(HIST_NOHIST,szFileName,bComposer);
          /* Positions initiales des pions */
     int aiPositions[10][10];
     if(Historique->Init(aiPositions)) {
          Plateau=new TPlateau(MainWindow, aiPositions);
          Plateau->EnableReserve(false);
          MainWindow->EnableReplayControl(true,true,true,true);
     } else {
          MainWindow->LoadError();
     }
}

__fastcall TPartie::~TPartie()
{ /* Destructeur */
     MainWindow->RemoveControl(Plateau);
     if(Plateau!=NULL) {
          delete Plateau;
          Plateau=NULL;
     }
     if(Historique!=NULL) {
          delete Historique;
          Historique=NULL;
     }
}

void __fastcall TPartie::EnregHistorique(bool bRougeInverse)
{ /* Enregistre la position initiale des pions dans le fichier de l'historique
     bRougeInverse sert  ne pas inverser le tableau dans le cas o on charge
     une partie o c'est au Rouge de jouer */
     if(Historique->GetType()==HIST_ENREG) {
          int aiPositions[10][10];
          Plateau->EnregHistorique(aiPositions, bRougeInverse);
          Historique->Positions(aiPositions);
     }
}

void __fastcall TPartie::PlacerPieces(bool bCouleur)
{ /* Place les pices dans la rserve,  droite de la carte */
     bJoueurActuel=bCouleur;
     MainWindow->EnableButtons(true,bCouleur,true,false,false,false);
     switch(bCouleur) {
          case JOUEUR_ROUGE :
               if(bRougePlaces==false) {
                    Plateau->PlacerReserve(bCouleur);
               } break;
          case JOUEUR_BLEU :
               if(bBleuPlaces==false) {
                    Plateau->PlacerReserve(bCouleur);
               } break;
     }
}

bool __fastcall TPartie::ValiderPieces()
{ /* Appel lors du clic sur le bouton "valider" */
  /* Retourne faux si le placement des pices n'est pas valide, vrai si
     on peut passer au joueur suivant ou commencer la partie */
     switch(bComposer) {
          case false :
               switch(Plateau->ValiderPieces()) {
                    case false : return false;
                    case true :
                         SetMode(MODE_NOMOVE);
                         if(bRougePlaces==false)
                              /* Le rouge n'a pas encore plac ses pices,
                                   alors c'est  lui de le faire */
                              MainWindow->EnableButtons(true,JOUEUR_ROUGE,
                                   true,false,false,false);
                         else if(bBleuPlaces==false)
                              /* Idem avec le bleu */
                              MainWindow->EnableButtons(true,JOUEUR_BLEU,
                                   true,false,false,false);
                         else
                              /* Sinon on pourra commencer la partie */
                              MainWindow->EnableButtons(true,false,
                                   false,false,false,true);
                         return true;
               }
               break;
          case true :
               if(Plateau->ValiderPieces()==true) {
                    /* En mode "composer" il suffit que les drapeaux soient
                         placs. Si c'est le cas, on supprime les pices qui
                         restent dans la rserve */
                    Plateau->VideReserve();
                    return true;
               }
               else return false;
     }
     return false;
}

void __fastcall TPartie::PlacerPiecesAuto(bool bOK)
{ /* Demande de placement automatique des pices */
  /* bOK = true si on avait cliqu sur "valider", = false si on avait
     simplement demander de placer les pices */
     Plateau->PlacerPiecesAuto(bJoueurActuel);
     if(bOK==true) PlacerPiecesOK();
}

void __fastcall TPartie::PlacerPiecesOK()
{ /* Verrouille le placement des pices */
     switch(bJoueurActuel) {
          case JOUEUR_ROUGE : bRougePlaces=true; break;
          case JOUEUR_BLEU  : bBleuPlaces =true; break;
     }
     SetMode(MODE_NOMOVE);
     if(bRougePlaces==true) {
          if(bBleuPlaces==true)
               MainWindow->EnableButtons(true,false,false,false,
                                                  false,true);
          else MainWindow->EnableButtons(true,JOUEUR_BLEU,true,
                                                  false,false,false);
     } else MainWindow->EnableButtons(true,JOUEUR_ROUGE,true,
                                                  false,false,false);
}

void __fastcall TPartie::Go()
{ /* Dmarre la partie */
     Plateau->EnableReserve(false);
     MainWindow->EnableStats(true);
     MainWindow->EnableSauver(bComposer);
     MainWindow->EnableButtons(false,false,false,false,false,false);
     SetMode(MODE_JEULOCAL);
     EnregHistorique(false);
}

void __fastcall TPartie::Go2()
{ /* Appel par l'cran intermdiaire en mode "composer" pour ventuellement
     commencer la partie si les deux joueurs ont plac leurs pices */
     if(bRougePlaces==true&&bBleuPlaces==true) Go();
}

void __fastcall TPartie::JoueurSuivant()
{ /* Passe au joueur suivant. Affiche l'cran intermdiaire et masque les
     pices si ncessaire */
     bJoueurActuel=!bJoueurActuel;
     if(bComposer==false) {
          Plateau->ChangeCote(bJoueurActuel);
     } else {
          Plateau->RetournePions(bJoueurActuel);
          if(Plateau->GetReserve()==false) {
               SetMode(MODE_JEULOCAL);
          }
     }
     if(Plateau->GetReserve()==true) {
          if(bRougePlaces==true&&bBleuPlaces==true) {
               MainWindow->EnableButtons(true,false,false,false,false,true);
          } else PlacerPieces(bJoueurActuel);
     }
}

void __fastcall TPartie::SetMode(int iNewMode)
{ /* Change le mode de jeu */
     Plateau->SetMode(iNewMode);
}

int __fastcall TPartie::GetMode()
{ /* Retourne le mode de jeu */
     return Plateau->GetMode();
}

void __fastcall TPartie::ComptePieces(int aiPieces[6][12])
{ /* Lance le comptage des pices */
     Plateau->ComptePieces(aiPieces);
}

bool __fastcall TPartie::Save(char *szFileName, bool bAffichePrises)
{ /* Lance la sauvegarde */
     return Plateau->Save(szFileName, bAffichePrises, bJoueurActuel);
}

bool __fastcall TPartie::NextMove()
{ /* Dplacement suivant lors du replay de l'historique */
     TDeplacement strDepl;
     if(Historique->NextMove(&strDepl)==false) return false;
     Plateau->NextMove(strDepl);
     return true;
}

void __fastcall TPartie::RemoveChangeJoueur()
{ /* Fait disparatre l'cran intermdiaire */
     Plateau->RemoveChangeJoueur(bJoueurActuel);
}

bool __fastcall TPartie::GetComposer()
{ /* Mode "composer" ou non */
     return bComposer;
}

// FIN <Pieces.cpp> ============================================================

