#ifndef PIECES_H
#define PIECES_H
/************************************************************************
 *                                                                      *
 *   Ant Stratego                                                       *
 *   (C) 2000 Antoine Potten                                            *
 *   software@antp.be - http://www.antp.be/software                     *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 2     *
 *   of the License, or (at your option) any later version.             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   pieces.h : Dfinition des classes Pion, Plateau, Partie et         *
 *              ChangeJoueur                                            *
 *                                                                      *
 ************************************************************************/

#include <vcl.h>
#include <Graphics.hpp>
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>
#include <stdio.h>
#include <string.h>
#include "Files.h"
#include "Hist.h"
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>

// lments de jeu
#define TOOLBAR_HEIGHT 30
#define STATUSBAR_HEIGHT 22
#define BUTTONS_HEIGHT 25
#define PIECE_HEIGHT 43
#define PIECE_WIDTH 43
#define PLATEAU_TOP 9
#define PLATEAU_LEFT 8
#define PLATEAU_HEIGHT 444
#define PLATEAU_WIDTH 444
#define PLATEAU_RESERVE_WIDTH 624

// joueurs
#define PION_VIDE 0
#define PION_ROUGE 1
#define PION_BLEU 2
#define JOUEUR_ROUGE true
#define JOUEUR_BLEU false

// modes de jeu
#define MODE_NOMOVE 0
#define MODE_RESERVE 1
#define MODE_JEULOCAL 2
#define MODE_JEURESEAU 3
#define MODE_REPLAY 4

#define HIST_NOHIST 0
#define HIST_ENREG  1
#define HIST_DEMAND 2


// TPion =======================================================================

class TPion : public TImage
{ // Classe drive de TImage
  private :
     int iRang;             // Rang de la pice (grade)
     int iCouleur;          // Couleur du pion
     TComponent *Plateau;   // Pointeur vers le plateau (parent)

  public :
     __fastcall TPion(TComponent* Owner, int, int, int, int);
     void __fastcall Deplace(int,int);
     void __fastcall Inverse(bool, bool);
     int __fastcall GetRang();
     int __fastcall GetCouleur();
     DYNAMIC void __fastcall DoEndDrag(TObject* Target, int X, int Y);
     DYNAMIC void __fastcall DragOver(TObject *Source, int X, int Y,
          TDragState State, bool &Accept);
};

// TChangeJoueur ===============================================================

class TChangeJoueur : public TImage
{ // Classe drive de TImage
  private :
     TComponent *Plateau; // Pointeur vers le plateau (parent)
     bool bJoueurActuel;  // Joueur actuel (dtermine le .bmp  utiliser)
     bool bComposer;      // Mode "composer" ou non
  public :
     DYNAMIC void __fastcall Click(void);
     __fastcall TChangeJoueur(TComponent* Owner,bool,bool);
     __fastcall ~TChangeJoueur();
     void __fastcall Affiche();
};

// TPlateau ====================================================================

class TPlateau : public TImage
{ // Classe drive de TImage
  private :
     int iMode;                          // Mode de jeu
     bool bReserve;                      // Rserve visible ou non
     bool bComposer;                     // Mode "composer" ou non
     TPion *Cases[14][10];               // Pointeurs vers les pions affichs
     TPion *Elimines[80];                // Pointeurs vers les pions limins
     int iNextElimine;                   // Nombre de pions limins
     TChangeJoueur *ChangeJoueurRouge;   // cran intermdiaire - joueur rouge
     TChangeJoueur *ChangeJoueurBleu;    //                     - joueur bleu
     TSavedGameEnreg strSavedGame;       // Structure du fichier pour la
  public :                               //                         sauvegarde
     void __fastcall RemoveChangeJoueur(bool);
     void __fastcall SetMode(int);
     int __fastcall GetMode();
     void __fastcall EnableReserve(bool);
     void __fastcall VideReserve();
     bool __fastcall GetReserve();
     bool __fastcall PlacerReserve(bool);
     void __fastcall PlacerPiecesAuto(bool);
     int __fastcall Deplace(int, int, int, int, bool);
     void __fastcall Elimine(int,int);
     void __fastcall Combat(int,int,int,int);
     int __fastcall Vainqueur(int,int);
     bool __fastcall ValiderPieces();
     void __fastcall ChangeCote(bool);
     void __fastcall RetournePions(bool);
     __fastcall TPlateau(TComponent* Owner, bool);
     __fastcall TPlateau(TComponent* Owner, char *, bool *, bool *);
     __fastcall TPlateau(TComponent *Owner, int [10][10]);
     __fastcall ~TPlateau();
     void __fastcall ComptePieces(int [6][12]);
     DYNAMIC void __fastcall DragOver(TObject *Source, int X, int Y,
               TDragState State, bool &Accept);
     bool __fastcall Save(char *, bool, bool);
     bool __fastcall Load(char *, bool *, bool *);
     void __fastcall EnregHistorique(int [10][10], bool);
     void __fastcall NextMove(TDeplacement);
     void __fastcall CachePions();
     void __fastcall AffichePions();
};

// TPartie =====================================================================

class TPartie
{
  private :
     TPlateau *Plateau;     // Plateau de jeu
     bool bJoueurActuel;    // Joueur actuel
     bool bRougePlaces;     // vrai si le joueur rouge a dj plac ses pions
     bool bBleuPlaces;      // vrai si le joueur bleu a dj plac ses pions
     bool bComposer;        // Mode "composer" ou non
     void __fastcall EnregHistorique(bool);
  public :
     void __fastcall RemoveChangeJoueur();
     THistorique *Historique;    // Historique
     __fastcall TPartie();
     __fastcall TPartie(bool);
     __fastcall TPartie(char *);
     __fastcall TPartie(int, char *);
     void __fastcall PlacerPieces(bool);
     bool __fastcall ValiderPieces();
     void __fastcall PlacerPiecesAuto(bool);
     void __fastcall PlacerPiecesOK();
     void __fastcall Go();
     void __fastcall Go2();
     void __fastcall JoueurSuivant();
     void __fastcall SetMode(int);
     int __fastcall GetMode();
     void __fastcall ComptePieces(int [6][12]);
     __fastcall ~TPartie();
     bool __fastcall Save(char *, bool);
     bool __fastcall NextMove();
     bool __fastcall GetComposer();
};

// FIN <Pieces.h> ==============================================================
#endif

