/************************************************************************
 *                                                                      *
 *   Ant Stratego                                                       *
 *   (C) 2000 Antoine Potten                                            *
 *   software@antp.be - http://www.antp.be/software                     *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 2     *
 *   of the License, or (at your option) any later version.             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   unit2.cpp : Options                                                *
 *                                                                      *
 ************************************************************************/

#include <vcl.h>
#include <vcl\inifiles.hpp> // Objet "ini" (gestion d'un fichier .ini)
#pragma hdrstop

#include "Unit2.h"
#include "Unit3.h"
//---------------------------------------------------------------------
#pragma resource "*.dfm"
TOptionsDLG *OptionsDLG;
//---------------------------------------------------------------------
__fastcall TOptionsDLG::TOptionsDLG(TComponent* AOwner)
	: TForm(AOwner)
{
}
//---------------------------------------------------------------------

bool __fastcall TOptionsDLG::Change()
{ /* Affiche la fentre permettant de changer les options, et initialise les
     diffrents "contrles" de cette fentre avec les valeurs prsentes en
     mmoire */
  /* Retourne vrai si on a cliqu sur OK, faux si on a cliqu Annuler */
     if(MainWindow->GetToolbarStyle()) Toolbar_Flat->Checked=true;
     else Toolbar_Normal->Checked=true;
     CBLogo->Checked=MainWindow->GetShowLogo();
     CBConfirmExit->Checked=MainWindow->GetConfirmExit();
     char *szReplayFile=NULL;
     switch(MainWindow->GetReplayStyle(&szReplayFile)) {
          case HIST_NOHIST : RBPasReplay->Checked=true; break;
          case HIST_ENREG  : RBEnreg->Checked=true; break;
          case HIST_DEMAND : RBDemander->Checked=true; break;
     }
     EFichierHistorique->SetTextBuf(szReplayFile);
     delete szReplayFile;
     TrackVitesse->Position=MainWindow->GetReplaySpeed();
     char szBuffer[8];
     sprintf(szBuffer,"%.2f",(TrackVitesse->Position)/20.0);
     LNbSecondes->SetTextBuf(szBuffer);
     CBRecentrer->Checked=MainWindow->GetRecentrer();
     return(OptionsDLG->ShowModal());
}

void __fastcall TOptionsDLG::ChargeOptions()
{  /* Relit les options du fichier .ini et les place en mmoire */
     SetCurrentDir(MainWindow->ASDirectory);
     TIniFile *ini;
     ini = new TIniFile(MainWindow->ASDirectory+"\\ant_stratego.ini");
     MainWindow->ChangeToolbarStyle(ini->ReadBool("Interface",
          "ToolbarStyle",true));
     AnsiString strReplayFile(ini->ReadString("Jeu","ReplayFile",""));
     MainWindow->ChangeReplayStyle(ini->ReadInteger("Jeu","ReplayStyle",
          HIST_DEMAND),strReplayFile.c_str());
     MainWindow->ChangeShowLogo(ini->ReadBool("Interface","ShowLogo",true));
     MainWindow->ChangeConfirmExit(ini->ReadBool("Interface",
          "ConfirmExit",true));
     MainWindow->ChangeReplaySpeed(ini->ReadInteger("Jeu","ReplaySpeed",5));
     MainWindow->ChangeRecentrer(ini->ReadBool("Interface","Recentrer",false));
     delete ini;
}

void __fastcall TOptionsDLG::SauveOptions()
{  /* Sauve les valeurs choisies  l'aide des diffrents contrles de la fentre
      Ces valeurs sont sauves en mmoire ainsi que dans le fichier .ini */
     MainWindow->ChangeToolbarStyle(Toolbar_Flat->Checked);
     MainWindow->ChangeShowLogo(CBLogo->Checked);
     MainWindow->ChangeConfirmExit(CBConfirmExit->Checked);
     MainWindow->ChangeReplaySpeed(TrackVitesse->Position);
     MainWindow->ChangeRecentrer(CBRecentrer->Checked);
     char szReplayFile[256];
     EFichierHistorique->GetTextBuf(szReplayFile,255);
     if(RBEnreg->Checked==true)
          MainWindow->ChangeReplayStyle(HIST_ENREG,szReplayFile);
     else if(RBDemander->Checked==true)
          MainWindow->ChangeReplayStyle(HIST_DEMAND,szReplayFile);
     else if(RBPasReplay->Checked==true)
          MainWindow->ChangeReplayStyle(HIST_NOHIST,szReplayFile);
     SetCurrentDir(MainWindow->ASDirectory);
     TIniFile *ini;
     ini = new TIniFile(MainWindow->ASDirectory+"\\ant_stratego.ini");
     ini->WriteBool("Interface","ToolbarStyle",Toolbar_Flat->Checked);
     ini->WriteBool("Interface","ShowLogo",CBLogo->Checked);
     ini->WriteBool("Interface","ConfirmExit",CBConfirmExit->Checked);
     char *szFileName=NULL;
     ini->WriteInteger("Jeu","ReplayStyle",
               MainWindow->GetReplayStyle(&szFileName));
     ini->WriteString("Jeu","ReplayFile",szFileName);
     ini->WriteInteger("Jeu","ReplaySpeed",TrackVitesse->Position);
     ini->WriteBool("Interface","Recentrer",CBRecentrer->Checked);
     delete szFileName;
     delete ini;
}
void __fastcall TOptionsDLG::BOKClick(TObject *Sender)
{    /* Lorsque l'on clique sur OK, applique et sauve les valeurs choisies */
     SauveOptions();
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDLG::BDefautClick(TObject *Sender)
{ /* Restaure les options par dfaut, mais ne les applique pas :
     seules les valeurs des contrles sont changes */
     Toolbar_Flat->Checked=true;
     CBLogo->Checked=true;
     CBConfirmExit->Checked=true;
     CBRecentrer->Checked=false;
     EFichierHistorique->SetTextBuf("");
     RBDemander->Checked=true;
     TrackVitesse->Position=20;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDLG::BOpenHistoriqueClick(TObject *Sender)
{ /* Choisit un fichier servant  stocker l'historique */
     if(ODParcourir->Execute()) {
          EFichierHistorique->SetTextBuf(ODParcourir->FileName.c_str());
     }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDLG::TrackVitesseChange(TObject *Sender)
{ /* Modifie le nombre de secondes affiches lorsqu'on change la valeur 
     l'aide du "curseur" (barre horizontale o l'on dplace une reglette */
     char szBuffer[8];
     sprintf(szBuffer,"%.2f",(TrackVitesse->Position)/20.0);
     LNbSecondes->SetTextBuf(szBuffer);
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDLG::BAideClick(TObject *Sender)
{ /* Affichage des rubriques d'aide */
     ShellExecute(0, NULL, "start", (ExtractFilePath(Application->ExeName)+"hlp\\index_options.html").c_str(), NULL, SW_NORMAL);
}
//---------------------------------------------------------------------------

